_G.FoSho = _G.FoSho or {}
FoSho.options_path = SavePath .. "Narrator Control.txt"
FoSho.options = {} 
FoSho.mod_path = ModPath
FoSho.locke_chance = 50

Hooks:Add("LocalizationManagerPostInit", "FoSho_Localization", function(loc)
	local loc_path = FoSho.mod_path .. "loc/"

	if file.DirectoryExists(loc_path) then
		for _, filename in pairs(file.GetFiles(loc_path)) do
			local str = filename:match('^(.*).json$')
			if str and Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
				loc:load_localization_file(loc_path .. filename)
				break
			end
		end
		loc:load_localization_file(loc_path .. "english.json", false)
	else
		log("Localization folder seems to be missing!")
	end
end)

function FoSho:Save()
	local file = io.open(self.options_path, "r")
	local temp_options = {}
	if file then
		temp_options = json.decode(file:read("*all"))
	end
	
	for id, data in pairs(self.options) do
		temp_options[id] = data
	end
	temp_options["locke_chance"] = FoSho.locke_chance
	file = io.open(self.options_path, "w+" )
	if file then
		file:write(json.encode(temp_options))
		file:close()
	end
end

function FoSho:Load()
	local file = io.open(self.options_path, "r")
	if file then
		local temp_options = json.decode( file:read("*all") )
		file:close()
		
		if temp_options["locke_chance"] then
			self.locke_chance = temp_options["locke_chance"]
			temp_options["locke_chance"] = nil
		end
		
		for k,v in pairs(temp_options) do 
			self.options[k] = v 
		end
		temp_options = nil
	else
		FoSho:Save()
	end
end

function FoSho:Stage_Narrator(stage_name)
	local file = io.open(self.options_path, "r")
	if file then
		local temp_options = json.decode(file:read("*all"))
		file:close()
		if temp_options[stage_name] then
			return temp_options[stage_name]
		end
		temp_options = nil
		return nil
	end
end

Hooks:Add("MenuManagerInitialize", "MenuManagerInitialize_FoSho", function(menu_manager)
	MenuCallbackHandler.locke_option_clbk = function(self, item)
		FoSho.locke_chance = item:value()
	end
	
	MenuCallbackHandler.locke_stages_clbk = function(self, item)
		FoSho.options[item:parameters().name] = item:value()
	end	
	
	MenuCallbackHandler.locke_force_narr_clbk = function(self, item)
		if managers.dialog then 
			managers.dialog:set_narrator("bain", item:value())
		end
	end	
	
	MenuCallbackHandler.locke_set_stages_clbk = function(self, item)
		local item_name = item:parameters().name
		local value = 3
		if item_name == "setall_bain" then
			value = 1
		elseif item_name == "setall_locke" then
			value = 2
		end
		local menu = MenuHelper:GetMenu("FoSho_menu")._items_list
		for _, item in pairs(menu) do
			if item._type == "multi_choice" and item:parameters().name ~= "current_narrator" then
				item:set_value(value)
				FoSho.options[item:parameters().name] = value
			end
		end
	end	
	
	MenuCallbackHandler.FoSho_save = function(self, item)
		FoSho:Save()
		FoSho.options = {}
	end
	
end)

Hooks:Add("MenuManagerSetupCustomMenus", "MenuManagerSetup_FoSho", function( menu_manager, nodes )
	MenuHelper:NewMenu("FoSho_menu")
end)

Hooks:Add("MenuManagerBuildCustomMenus", "MenuManagerBuildCustomMenus_FoSho", function( menu_manager, nodes )

	local data = {
		focus_changed_callback = nil,
		back_callback = "FoSho_save",
		area_bg = nil,
	}
	nodes["FoSho_menu"] = MenuHelper:BuildMenu("FoSho_menu", data)
	local menu_priority = nil
	if menu_priority ~= nil then
		for k, v in pairs( nodes["blt_options"]._items ) do
			if menu_priority > (v._priority or 0) then
				menu_priority = k
				break
			end
		end
	end
	MenuHelper:AddMenuItem(nodes["blt_options"], "FoSho_menu", "FoSho_options_title", "FoSho_options_desc", menu_priority)
end)

Hooks:Add("MenuManagerPopulateCustomMenus", "MenuManagerPopulateCustomMenus_FoSho", function( menu_manager, nodes )
	FoSho:Load()
	local track_list = {}
	local unique_jobs = {}
	
	for _, job_id in ipairs(tweak_data.narrative:get_jobs_index()) do
		local job_tweak = tweak_data.narrative:job_data(job_id)
		if not table.contains(unique_jobs, job_tweak.name_id) then
			local name_id = managers.localization:to_upper_text(job_tweak.name_id)
			local days = #tweak_data.narrative:job_chain(job_id)
			local job_chain = tweak_data.narrative:job_chain(job_id)
			table.insert(unique_jobs, job_tweak.name_id)
			for i = days, 1, -1 do
				local level_id = job_chain[i].level_id
				local level_name = tweak_data.levels[level_id] and tweak_data.levels[level_id].name_id
				local id = job_tweak.name_id .. (days > 1 and i or "")
				table.insert(track_list, {
					desc = level_name,
					name = name_id .. (days > 1 and ": " .. managers.localization:text("menu_jukebox_heist_day", {day = i}) or ""),
					sort_id = name_id,
					id = id,
				})
			end
		end
	end
	table.sort(track_list, function (x, y)
			return x.sort_id == y.sort_id and x.name > y.name or x.sort_id > y.sort_id
	end)
	for id, level in pairs(track_list) do
		if level.name then
			MenuHelper:AddMultipleChoice({
				id = level.id,
				title = level.name,
				desc = level.desc,
				callback = "locke_stages_clbk",
				items = {"heist_contact_bain", "heist_contact_locke", "FoSho_random"},
				value = FoSho.options[level.id] or 3,
				menu_id = "FoSho_menu",
				priority = id,
				localized = false,
			})
		end
	end
	MenuHelper:AddDivider({
		id = "divider_FoSho_menu_" .. tostring(#track_list + 1),
		size = 15,
		menu_id = "FoSho_menu",
		priority = #track_list + 1,
	})
	MenuHelper:AddButton({
		id = "setall_random",
		title = "FoSho_setall_random_title",
		desc = "FoSho_setall_random_desc",
		callback = "locke_set_stages_clbk",
		next_node = nil,
		menu_id = "FoSho_menu",
		priority = #track_list + 2,
		localized = true,
	})
	MenuHelper:AddButton({
		id = "setall_locke",
		title = "FoSho_setall_locke_title",
		desc = "FoSho_setall_locke_desc",
		callback = "locke_set_stages_clbk",
		next_node = nil,
		menu_id = "FoSho_menu",
		priority = #track_list + 3,
		localized = true,
	})
	MenuHelper:AddButton({
		id = "setall_bain",
		title = "FoSho_setall_bain_title",
		desc = "FoSho_setall_bain_desc",
		callback = "locke_set_stages_clbk",
		next_node = nil,
		menu_id = "FoSho_menu",
		priority = #track_list + 4,
		localized = true,
	})
	MenuHelper:AddButton({
		id = "FoSho_stages",
		title = "FoSho_stages_title",
		desc = "FoSho_stages_desc",
		callback = nil,
		next_node = nil,
		menu_id = "FoSho_menu",
		priority = #track_list + 5,
		localized = true,
	})
	MenuHelper:AddDivider({
		id = "divider_FoSho_menu_" .. tostring(#track_list + 5),
		size = 25,
		menu_id = "FoSho_menu",
		priority = #track_list + 6,
	})
	MenuHelper:AddSlider({
		id = "locke_chance",
		title = "FoSho_locke_chance_title",
		desc = "FoSho_locke_chance_desc",
		callback = "locke_option_clbk",
		value = FoSho.locke_chance,
		min = 0,
		max = 100,
		step = 0.01,
		show_value = true,
		menu_id = "FoSho_menu",
		priority = #track_list + 7,
		localized = true,
	})
	MenuHelper:AddMultipleChoice({
		id = "current_narrator",
		title = "FoSho_current_title",
		desc = "FoSho_current_desc",
		callback = "locke_force_narr_clbk",
		items = {"heist_contact_bain", "heist_contact_locke", "FoSho_none"},
		value = 3,
		menu_id = "FoSho_menu",
		priority = #track_list + 8,
		localized = true,
	})
	local menu = MenuHelper:GetMenu("FoSho_menu")._items_list
	if menu and menu[#menu]._type == "multi_choice" then
		menu[#menu]:set_enabled(false)
	end
	track_list = nil
	unique_jobs = nil
	FoSho.options = {}
end)